#
#       HBS REDUCTION SUB ROUTINE
#
#       make one-dimensional spectra
#                                               : hbs1d.cl
#
#       1997/08/09      K.Kawabata  Ver.0.50  original 1d routine
#       1998/02/09      K.Kawabata  Ver.1.00  
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#       1998/05/31      K.Kawabata  Ver.2.10
#       2000/02/29      K.Kawabata  Ver.3.00  for SITe-CCD
#

procedure hbs1d

begin

#  file fn_postident = "dispref.od.0"
  file fn_ypeak
  string rdir
  file tmp_list, fn_last
  file fn, fnnexte, fnnexto
  file fnobe, fnobo, fnobe1, fnobo1
  string fn_stack[500]
  int n_image
  file logfn
  int x, c, i
  string hbslist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  string listoflist
  int n_list
  int sp1, sp2;

  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7
  int flag_uplow  #  0:lower  1: upper  -1:other

  int flag_erase # 0:keep old images, 1:erase old images
  int flag_inquiry = 0   # 1:'flag_overwrite' applys to all image
  int flag_overwrite = 0 # 0:skip 1:overwrite
  int flag_ver       # 0:no display 1:display

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "----------------------------" )
  print( " scan and extract apertures" )
  print( "----------------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "----------------------------", >> logfn )
  print( " scan and extract apertures", >> logfn )
  print( "----------------------------", >> logfn )
  print( "", >> logfn )

  buf = ""
  listoflist = envget( "_rootlist" )
  rdir = envget( "_caldir" )
  fn_ypeak = envget( "_ysftlog" )

  tmp_list = mktemp( "hbs.tmp.list." )

  if( access( fn_ypeak ) == 1 )
    delete( fn_ypeak, ver- )

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    print( "Cannot find "//listoflist//"   Please do hbsgroup", >> logfn )
    bye
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n" )
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n",\
                >> logfn )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del2" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del2" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_del2) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_del2) Retry.\n", >> logfn )
      bye
    }
  }

#----- Scanning the list file

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL",\
             >> logfn )
      break
    }
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- main routine

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" )
    {

#----- check diaphragm

      c = strlen( diaph[i] )
      if( substr( diaph[i], 4, 4 ) == "L" )
      {
        flag_uplow = 0
        if( flag_ver == 1 )
          printf( "--- OBJECT LIST : %s  LOWER ---\n",\
	          hbslist[i] )
        printf( "--- OBJECT LIST : %s  LOWER ---\n",\
	        hbslist[i], >> logfn )
      }
      else
      {
        if( substr( diaph[i], 4, 4 ) == "U" )
        {
          flag_uplow = 1
          if( flag_ver == 1 )
            printf( "--- OBJECT LIST : %s  UPPER ---\n",\
	            hbslist[i] )
          printf( "--- OBJECT LIST : %s  UPPER ---\n",\
	          hbslist[i], >> logfn )
        }
        else
        {
          flag_uplow = -1
          printf( "%s did not used two diaphragm mode\n", hbslist[i] )
          printf( "%s did not used two diaphragm mode\n", hbslist[i],\
                  >> logfn )
	  next
        }
      }

#----- extract OBJECT spectra

      sed( 's/fit\.imh//', hbslist[i], > tmp_list )

      n_image = 0
      list = tmp_list
      while( fscan( list, fn ) != EOF )
      {
        n_image += 1
        fn_stack[n_image] = fn
      }

      for( j=1; j<=n_image; j+=1 )
      {
        fn = fn_stack[j]
	
        if( flag_uplow == 0 )
	{
          fnobe1 = fn//"fl.3"
          fnobo1 = fn//"fl.4"
          fnobe = fn//"od.e"
          fnobo = fn//"od.o"
	  sp1 = 3
	  sp2 = 4
	}
	else
	{
          fnobe1 = fn//"fl.1"
          fnobo1 = fn//"fl.2"
          fnobe = fn//"od.e"
          fnobo = fn//"od.o"
	  sp1 = 1
	  sp2 = 2
	}
        if( j < n_image )
        {
          fnnexte = fn_stack[j+1]//"od.e"
          fnnexto = fn_stack[j+1]//"od.o"
	}
	else
	{
          fnnexte = "none"
          fnnexto = "none"
	}

        if( access( fnobe1//".imh" ) == 0 || access( fnobo1//".imh" ) == 0 )
        {
          printf( "%s--(not found. skip.)  ", fn )
          printf( "%s--(not found. skip.)  ", fn, >> logfn )
          next
        }

        if( access( fnobe//".imh" ) == 1 || access( fnobo//".imh" ) == 1 )
        {
          if( flag_overwrite == 0 )
          {
            if( fnnexte != "none" && ( access( fnnexte//".imh" ) == 0 ||\
	        access( fnnexto//".imh" ) == 0 ) )
            {
	      printf( "\n%s: Maybe insufficient proccess last time.\n", fn )
	      printf( "\n%s: Maybe insufficient proccess last time.\n",\
                      fn, >> logfn )
              goto inquiry
            }

            if( flag_inquiry == 1 )
            {
              if( flag_ver == 1 )
                printf( "%s--(already exists. skip.)  ", fn )
              printf( "%s--(already exists. skip.)  ", fn, >> logfn )
              next
            }
            else
	    {
              printf( "\n%s already exist.\n", fn )
              printf( "\n%s already exist.\n", fn, >> logfn )
          inquiry:
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> ", >> logfn )
              c = scan( buf )
              while( buf != "Y" && buf != "y" && buf != "n" && buf != "N" )
                c = scan( buf )
              printf( "%s\n", buf, >> logfn )
              if( buf == "Y" )
              {
                flag_inquiry = 1
                flag_overwrite = 1
              }
              if( buf == "N" )
              {
                flag_inquiry = 1
                flag_overwrite = 0
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn )
                printf( "%s--(already exists. skip.)  ", fn, >> logfn )
                next
              }
              if( buf == "n" )
              {
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn )
                printf( "%s--(already exists. skip.)  ", fn, >> logfn )
                next
              }
            }
          }
          if( flag_ver == 1 )
            printf( "%s(overwrite)  ", fn )
          printf( "%s(overwrite)  ", fn, >> logfn )
        }
        else
        {
          if( flag_ver == 1 )
            printf( "%s  ", fn )
          printf( "%s  ", fn, >> logfn )
        }

        hbsap1d( fnobe1, fnobo1, sp1, sp2, fn_ypeak, fnobe, fnobo )

	if( access( fnobe//".imh" ) == 0 || access( fnobe//".imh" ) == 0 )
	{
	  printf( "One-demensional image failed. Abnormal end." )
	  printf( "One-demensional image failed. Abnormal end.", >> logfn )
	  bye
	}
	
        imreplace( fnobe, 0.0, lower=INDEF, upper=0.0 )
        imreplace( fnobo, 0.0, lower=INDEF, upper=0.0 )
        imreplace( fnobe, 1.0e+25, lower=1.0e+25, upper=INDEF )
        imreplace( fnobo, 1.0e+25, lower=1.0e+25, upper=INDEF )

#	refspec( fnobe, references=fn_postident, ignoreaps+, time-,\
#	         override+, confirm-, assign+, sort="", group="",\
#		 logfiles="", ver- )
#        hedit( buf1, "HISTORY", "one-dimensional spectrum",\
#	       delete-, verify-, show-, update+ )

#	refspec( fnobo, references=fn_postident, ignoreaps+, time-,\
#	         override+, confirm-, assign+, sort="", group="",\
#		 logfiles="", ver- )
#        hedit( buf2, "HISTORY", "one-dimensional spectrum",\
#	       delete-, verify-, show-, update+ )

        if( flag_erase == 1 )
	{
	  imdelete( fnobe1, ver- )
	  imdelete( fnobo1, ver- )
	}
      }
      delete( tmp_list, ver- )
    }
    printf( "\n" )
    printf( "\n", >> logfn )
  }

end
