#
#       HBS REDUCTION SUB ROUTINE
#
#       Wavelength arrange/smoothed FLAT-2 : hbsflatarr.cl
#
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#

procedure hbsflatarr

begin

  file fn, fn1, fn2
  int i, c
  file flatlist1, flatlist2
  int n_list
  string hbsfile[50]
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8, buf9
  string caldir, rdir
  file fn_def
  real crpix, crval, cdelt

  file fni1, fni2, fni3, fni4
  file fno1, fno2, fno3, fno4
  file feature_fn
  string feat_stack[20]
  int n_feat
  real lmd10, lmd20, dl1, dl2, dl0
  real lmd1, lmd2
  real x1, x2, dx1, dx2, dx0
  int nx1, nx2
  string feattyp1, feattyp2, comm1, comm2
  file ref_fni = "hbs.ident.od"
  file ref_fnr = "hbs.refer.od"
  file shiftlog
  int dy

  int nfeat = 2
  real os, os_2HL, os_2S2, os_tmp
  int nccd
  
  file tmp_list, tmp_fn, tmp_tbl
  file tmp_fn1 
  int runmeanf  = 200   # boxcar factor for identify
  int multiplef = 10000 # multiple factor for identify
  int fwidth1
  string fitysection = "middle line"
  real threshold1 = 2.0
  real crad1 = 30.0
  string nullfile
  real secppix, seeing
  int flag_first     # for identify-refeerence
  int n_version      # IRAF version (210 or 211)

  int ap1, ap2

#--- initial settings

  print( "" )
  print( "------------------------------------------------" )
  print( " Wavelength Arrange/Smoothing for FLAT-2 images" )
  print( "------------------------------------------------" )
  print( "" )

  buf = ""
  flatlist1 = "hbs.tmp.flat1"
  flatlist2 = "hbs.tmp.flat2"
  caldir = envget( "_caldir" )
  etcdir = envget( "_etcdir" )
  shiftlog = envget( "_xsftlog" )
  nullfile = envget( "_nullfile" )
  fn_prefit = caldir
#  secppix = real( envget( "_secppix" ) ) # deleted from SITe CCD version
  fn_def = caldir//envget( "_defhname" )
  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_tbl = "hbs.tmp.fxcor"
  tmp_fn = mktemp( "hbs.tmp.image." )
  tmp_fn1 = mktemp( "hbs.tmp.image." )

  if( access( ref_fni//".imh" ) == 1 )
    imdelete( ref_fni, ver- )
  if( access( ref_fnr//".imh" ) == 1 )
    imdelete( ref_fnr, ver- )
  if( access( "database/id"//ref_fni ) == 1 )
    delete( "database/id"//ref_fni, ver- )


#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    bye
  }

  crpix = 1.0
  crval = 0.0
  cdelt = 0.0
  os_2HL = 9999.0
  os_2S2 = 9999.0
  nccd = 1        # for old 'hbsdefine.h'

  secppix = 0.22  # from SITe-CCD version

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "SW" )
    {
      crval = real( buf3 )
    }
    if( buf2 == "DW" )
    {
      cdelt = real( buf3 )
    }
    if( buf2 == "OS_2HL" )
    {
      os_2HL = real( buf3 )
    }
    if( buf2 == "OS_2S2" )
    {
      os_2S2 = real( buf3 )
    }
    if( buf2 == "NCCD" )
      nccd = int( buf3 )
    if( buf2 == "ASPPIX" )
      secppix = real( buf3 )
  }
  if( ( crval * cdelt ) == 0 || os_2HL > 5000 || os_2S2 > 5000  )
  {
    printf( "Read error in '%s'\n", fn_def )
    bye
  }

#----- Scanning the FLAT list file

  if( access( flatlist1 ) == 1 )
    delete( flatlist1, ver- )
  if( access( flatlist2 ) == 1 )
    delete( flatlist2, ver- )

  sections( "FLAT.*.px.imh", option="fullname", >> flatlist1 )
  sed( "s/px\.imh//", flatlist1, >> flatlist2 )

  delete( flatlist1, ver- )

  n_list = 0
  list = flatlist2
  while( fscan(list, buf1) != EOF )
  {
    n_list += 1
    hbsfile[n_list] = buf1
  }

  delete( flatlist2, ver- )

#----- main routine

  flag_first = 1

  for( i=1; i<=n_list; i+=1 )
  {
    rdir = caldir

    fn = hbsfile[i]

    fn1 = fn
    fni1 = fn//"px.1"
    fni2 = fn//"px.2"
    fni3 = fn//"px.3"
    fni4 = fn//"px.4"
    fno1 = fn//"sf.1"
    fno2 = fn//"sf.2"
    fno3 = fn//"sf.3"
    fno4 = fn//"sf.4"

    if( access( fno1//".imh" ) == 1 )
      imdelete( fno1, ver- )
    if( access( fno2//".imh" ) == 1 )
      imdelete( fno2, ver- )
    if( access( fno3//".imh" ) == 1 )
      imdelete( fno3, ver- )
    if( access( fno4//".imh" ) == 1 )
      imdelete( fno4, ver- )

#----- Wavelength arrangement for FLAT2

    if( substr( fn, 1, 11 ) == envget( "_fl2hl2" ) || substr( fn, 1, 11 ) == envget( "_fl2s21" ) )
    {

      printf( "%s  ", fn )

  #----- shift width

    #----- Identify reference frame (only at first of each group)

      if( flag_first == 1 )
      {
	flag_first = 0
        if( access( ref_fni//".imh" ) == 1 &&\
	    access( "database/id"//ref_fni ) == 1 )
	{
	  printf( "Use previous identification? [y/n] >> " )
	  c = scan( buf )
	  while( buf != "n" && buf != "y" )
            c = scan( buf )
	  if( buf == "y" )
	    goto target
	}

        if( access( ref_fni//".imh" ) == 1 )
          imdelete( ref_fni, ver- )
        if( access( ref_fnr//".imh" ) == 1 )
          imdelete( ref_fnr, ver- )
        if( access( "database/id"//ref_fni ) == 1 )
          delete( "database/id"//ref_fni, ver- )

        imarith( fni1, "+", fni2, ref_fnr, pixtype="real",\
                 calctype="real", ver- )
        imgets( ref_fnr, 'i_naxis2' )
        dy = int( imgets.value )
        blkavg( ref_fnr, ref_fnr, 1, dy, option="sum" )
        boxcar( ref_fnr, tmp_fn1, runmeanf, 1, boundary="nearest" )
        imarith( ref_fnr, "/", tmp_fn1, ref_fni, pixtype="real",\
                 calctype="real", ver- )
        imdelete( tmp_fn1, ver- )
        imarith( ref_fni, "*", multiplef, ref_fni, pixtype="real",\
                 calctype="real", ver- )

	feature_fn = envget( "_featuren" )

      #-- read feature file

	comm1 = ""
	comm2 = ""
	list = feature_fn
	if( fscan( list, feattyp1, lmd10, dl1, comm1 ) != 4 )
	{
	  printf( "Read error '%s'\n", feature_fn )
	  bye
	}
	if( fscan( list, feattyp2, lmd20, dl2, comm2 ) != 4 )
	{
	  printf( "Read error '%s'\n", feature_fn )
	  bye
	}
	if( feattyp1 == "e" )
	  feattyp1 = "emission"
	if( feattyp1 == "a" )
	  feattyp1 = "absorption"
	if( feattyp2 == "e" )
	  feattyp2 = "emission"
	if( feattyp2 == "a" )
	  feattyp2 = "absorption"

	#--- TI CCD feature avoid

	if( nccd == 2 && feature_fn == envget( "_featuren" ) )
	{
	  nfeat = 1
          lmd10 = lmd20
	  dl1 = dl2
	  comm1 = comm2
	  feattyp1 = feattyp2
	}

      #-- first feature
	fwidth1 = dl1 / cdelt
	x1 = ( lmd10 - crval ) / cdelt + crpix
	printf( "Mark with 'm' near %s feature at X=%3.0f of %2.0f width\n",\
	        feattyp1, x1, fwidth1 )
	if( n_version == 210 )
	{
          identify( ref_fni, section=fitysection, databas="database",\
                    coordli=nullfile, nsum=1, match=30.,\
                    maxfeat=1, fwidth=fwidth1, cradius=crad1,\
	            functio="legendre", order=2, niterate=1,\
	            low_rej=3., high_rej=3., grow=0., ftype=feattyp1,\
		    mode="ql", threshold=threshold1, autowrite+ )
	}
	else
	{
	  if( n_version == 211 )
	  {
            identify( ref_fni, section=fitysection, databas="database",\
                      coordli=nullfile, nsum=1, match=30.,\
		      units="angstroms",\
                      maxfeat=1, fwidth=fwidth1, cradius=crad1,\
	              functio="legendre", order=2, niterate=1,\
	              low_rej=3., high_rej=3., grow=0., ftype=feattyp1,\
		      mode="ql", threshold=threshold1, autowrite+ )
	  }
	  else
	  {
            printf( "IRAF version erorr (%d)\n", n_version )
            bye
	  }
	}
        list = "database/id"//ref_fni
        c = fscan( list, buf, buf, buf, buf, buf, buf )
        c = fscan( list, buf, buf, buf, buf, buf, buf )
        while( fscan( list, buf1, buf2, buf3, buf, buf, buf ) != 6 ){}
        x1 = real( buf1 )
        lmd1 = crval + ( x1 - crpix ) * cdelt
	dl1 = lmd1 - lmd10
	dx1 = dl1 / cdelt
        printf( "Read:%7.2fA  Diff:%7.2fA = %5.2fpixels\n", lmd1,\
	        dl1, dx1 )
        delete( "database/id"//ref_fni, ver- )

        if( nfeat == 2 )
        {

      #-- second feature
	  fwidth1 = dl2 / cdelt
	  x2 = ( lmd20 - crval ) / cdelt + crpix
	 printf( "Mark with 'm' near %s feature at X=%3.0f of %2.0f width\n",\
	          feattyp2, x2, fwidth1 )
	  if( n_version == 210 )
	  {
            identify( ref_fni, section=fitysection, databas="database",\
                      coordli=nullfile, nsum=1, match=30.,\
                      maxfeat=1, fwidth=fwidth1, cradius=crad1,\
	              functio="legendre", order=2, niterate=1,\
	              low_rej=3., high_rej=3., grow=0., ftype=feattyp2,\
	              mode="ql", threshold=threshold1, autowrite+ )
	  }
	  else
	  {
	    if( n_version == 211 )
	    {
              identify( ref_fni, section=fitysection, databas="database",\
                        coordli=nullfile, nsum=1, match=30.,\
	 	        units="angstroms",\
                        maxfeat=1, fwidth=fwidth1, cradius=crad1,\
	                functio="legendre", order=2, niterate=1,\
	                low_rej=3., high_rej=3., grow=0., ftype=feattyp2,\
		        mode="ql", threshold=threshold1, autowrite+ )
	    }
	    else
	    {
              printf( "IRAF version erorr (%d)\n", n_version )
              bye
	    }
	  }
          list = "database/id"//ref_fni
          c = fscan( list, buf, buf, buf, buf, buf, buf )
          c = fscan( list, buf, buf, buf, buf, buf, buf )
          while( fscan( list, buf1, buf2, buf3, buf, buf, buf ) != 6 ){}
          x2 = real( buf1 )
          lmd2 = crval + ( x2 - crpix ) * cdelt
	  dl2 = lmd2 - lmd20
	  dx2 = dl2 / cdelt
          printf( "Read:%7.2fA  Diff:%7.2fA = %5.2fpixels\n", lmd2,\
	          dl2, dx2 )
          delete( "database/id"//ref_fni, ver- )

	  dx0 = ( dx1 + dx2 ) / 2.0
	  if( abs( dx1 - dx2 ) < 7.0 )
          {
	    printf( "Average = %5.2fpixels\n", dx0 )
	  }
	  else
	  {
	    printf( "Difference(=%5.2fpix) is not negligible. Maybe failure.\n",\
	            abs( dx1 - dx2 ) )
inputdx0:
	    printf( "Manual number input around %6.2f (pix) >> ", dx0 )
	    c  = scan( dx0 )
#	    if( abs( dx0 ) > 50 )
#	      goto inputdx0
	    printf( "Input value = %5.2fpixels\n", dx0 )
	  }
        }
	else
	{
	  dx0 = dx1
	}
      }

  target:

  #----- target Sp1/Sp2 (automatic cross-correlation)

      if( access( tmp_tbl//".txt" ) == 1 )
	delete( tmp_tbl//".txt", ver- )
      imarith( fni1, "+", fni2, tmp_fn, pixtype="real",\
               calctype="real", ver- )
      imgets( tmp_fn, 'i_naxis2' )
      dy = int( imgets.value )
      imgets( tmp_fn, 'i_naxis1' )
      nx1 = 0.15 * int( imgets.value )
      nx2 = 0.85 * int( imgets.value )
      blkavg( tmp_fn, tmp_fn, 1, dy, option="sum" )
      hbscoord( tmp_fn )
#      fxcor( tmp_fn, ref_fnr, continuum="none", filter="none",\
#	      pixcorr+, osample=nx1//":"//nx2, rsample=nx1//":"//nx2,\
#	      function="gaussian", apertures="*", cursor="",\
#	      width=21.0, weights=1.0, background=0.0, window=INDEF,\
#	      wincenter=INDEF, output=tmp_tbl, verbose="stxtonly",\
#	      imupdate-, interactive-, autodraw+ )
      fxcor( tmp_fn, ref_fnr,\
	     pixcorr+, osample=nx1//":"//nx2, rsample=nx1//":"//nx2,\
	     function="gaussian", weights=1.0,\
	     width=21.0, output=tmp_tbl, verbose="stxtonly",\
	     interactive- )

      if( access( tmp_tbl//".txt" ) == 0 )
      {
        printf( "Error in fxcor text1 for %s\n", fn )
	bye
      }
      list = tmp_tbl//".txt"
      c = fscan( list, buf1, buf2, buf3, buf4, buf5, buf6, buf7,\
                 buf8, buf9 )
      while( substr( buf1, 1, 1 ) == "#" )
        c = fscan( list, buf1, buf2, buf3, buf4, buf5, buf6, buf7,\
	           buf8, buf9 )
      if( c != 9 )
      {
        printf( "Error in fxcor text2 for %s\n", fn )
	bye
      }
      dx1 = real( buf6 )
      dx2 = dx0 + dx1
      printf( "%5.2f %5.2f %5.2f\n", dx0, dx1, dx2 )
      printf( "%s %5.2f %5.2f %5.2f\n", fn, dx0, dx1, dx2, >> shiftlog )
      imdelete( tmp_fn, ver- )
      delete( tmp_tbl//".txt", ver- )

  #---- Shift

      if( access( fno1//".imh" ) == 1 )
        imdelete( fno1, ver- )
      hbsshift( fni1, fno1, ( -dx2 ), 0.0 )
      hbscoord( fno1 )

      if( access( fno2//".imh" ) == 1 )
	imdelete( fno2, ver- )
      hbsshift( fni2, fno2, ( -dx2 ), 0.0 )
      hbscoord( fno2 )

  #---- FITS header comments

      hedit( fno1, "CRPIX1", crpix, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno1, "CRVAL1", crval, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno1, "CD1_1", cdelt, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno1, "CDELT1", cdelt, add+, delete-, verify-, show-,\
             update+ )

      hedit( fno2, "CRPIX1", crpix, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno2, "CRVAL1", crval, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno2, "CD1_1", cdelt, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno2, "CDELT1", cdelt, add+, delete-, verify-, show-,\
             update+ )


  #----- target Sp3/Sp4 (automatic cross-correlation)

      if( access( tmp_tbl//".txt" ) == 1 )
	delete( tmp_tbl//".txt", ver- )
      imarith( fni3, "+", fni4, tmp_fn, pixtype="real",\
               calctype="real", ver- )
      imgets( tmp_fn, 'i_naxis2' )
      dy = int( imgets.value )
      blkavg( tmp_fn, tmp_fn, 1, dy, option="sum" )
      hbscoord( tmp_fn )
#      fxcor( tmp_fn, ref_fnr, continuum="none", filter="none",\
#	      pixcorr+, osample="100:900", rsample="100:900",\
#	      function="gaussian", apertures="*", cursor="",\
#	      width=21.0, weights=1.0, background=0.0, window=INDEF,\
#	      wincenter=INDEF, output=tmp_tbl, verbose="stxtonly",\
#	      imupdate-, interactive-, autodraw+ )
      fxcor( tmp_fn, ref_fnr,\
	     pixcorr+, osample="100:900", rsample="100:900",\
	     function="gaussian", weights=1.0,\
	     width=21.0, output=tmp_tbl, verbose="stxtonly",\
	     interactive- )

      if( access( tmp_tbl//".txt" ) == 0 )
      {
        printf( "Error in fxcor text1 for %s\n", fn )
	bye
      }
      list = tmp_tbl//".txt"
      c = fscan( list, buf1, buf2, buf3, buf4, buf5, buf6, buf7,\
                 buf8, buf9 )
      while( substr( buf1, 1, 1 ) == "#" )
        c = fscan( list, buf1, buf2, buf3, buf4, buf5, buf6, buf7,\
	           buf8, buf9 )
      if( c != 9 )
      {
        printf( "Error in fxcor text2 for %s\n", fn )
	bye
      }
      dx1 = real( buf6 )
      dx2 = dx0 + dx1
      printf( "%5.2f %5.2f %5.2f\n", dx0, dx1, dx2 )
      printf( "%s %5.2f %5.2f %5.2f\n", fn, dx0, dx1, dx2, >> shiftlog )
      imdelete( tmp_fn, ver- )
      delete( tmp_tbl//".txt", ver- )

  #---- Shift

      if( access( fno3//".imh" ) == 1 )
        imdelete( fno3, ver- )
      hbsshift( fni3, fno3, ( -dx2 ), 0.0 )
      hbscoord( fno3 )

      if( access( fno4//".imh" ) == 1 )
	imdelete( fno4, ver- )
      hbsshift( fni4, fno4, ( -dx2 ), 0.0 )
      hbscoord( fno4 )

  #---- FITS header comments

      hedit( fno3, "CRPIX1", crpix, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno3, "CRVAL1", crval, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno3, "CD1_1", cdelt, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno3, "CDELT1", cdelt, add+, delete-, verify-, show-,\
             update+ )

      hedit( fno4, "CRPIX1", crpix, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno4, "CRVAL1", crval, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno4, "CD1_1", cdelt, add+, delete-, verify-, show-,\
             update+ )
      hedit( fno4, "CDELT1", cdelt, add+, delete-, verify-, show-,\
             update+ )

    }
  }
 
end
