#
#       HBS REDUCTION SUB ROUTINE
#
#     make interpolated image according to specified KEYWORD of header
#                             : hbsiminterp.cl
#
#       1997/04/30      Ver. 2.00    K.Kawabata
#

procedure hbsiminterp( infile1, keyin1, infile2, keyin2, logscale, outfile, keycon )

file infile1	{ prompt = "Input image 1" }
real  keyin1	{ prompt = "Interpolation Reference value of image 1" }
file infile2	{ prompt = "Input image 2" }
real  keyin2	{ prompt = "Interpolation Reference value of image 2" }
int logscale	{ prompt = "Interpolation with [0]:linear-scale [1]:log-scale" }
file outfile	{ prompt = "Output image" }
real  keycon	{ prompt = "Interpolation Reference value of output image" }

begin

  file infn1, infn2, outfn, fnave
  string beta1, beta2
  file tmp_fn1, tmp_fn2
  real xkey1, xkey2, xkeyave
  real xkey_tmp
  real keyval
  string buf1, buf2
  int i
  int flag_log # interpolation [0] with linear-scale [1] with log-scale

#----- initial settings

  fnave = mktemp( "hbs.tmp.image." )
  beta1 = mktemp( "hbs.tmp.image." )
  beta2 = mktemp( "hbs.tmp.image." )

  infn1 = infile1
  xkey1 = keyin1
  infn2 = infile2
  xkey2 = keyin2
  flag_log = logscale
  outfn = outfile
  keyval = keycon

#----- Reference Keyword

  if( access( infn1 ) != 1 && access( infn1//".imh" ) != 1 )
  {
    printf( "'%s' not found\n", infn1 )
    bye
  }
  if( access( infn2 ) != 1 && access( infn2//".imh" ) != 1 )
  {
    printf( "'%s' not found\n", infn2 )
    bye
  }

#----- For Avoiding numerical round effects

  xkey2 -= xkey1
  keyval -= xkey1
  xkey1 = 0.0

#----- Calculation of Coefficients

  if( flag_log == 0 )
  {
    tmp_fn1 = infn1
    tmp_fn2 = infn2
  }
  else
  {
    tmp_fn1 = mktemp( "hbs.tmp.image." )
    tmp_fn2 = mktemp( "hbs.tmp.image." )
    imfunc( infn1, tmp_fn1, function="log10", ver- )
    imfunc( infn2, tmp_fn2, function="log10", ver- )
  }

  imarith( tmp_fn1, "+", tmp_fn2, fnave, pixtype="real", calctype="real",\
           ver- )
  imarith( fnave, "/", 2, fnave, pixtype="real", calctype="real",\
           ver- )
  xkeyave = ( xkey1 + xkey2 ) / 2.0

  buf1 = mktemp( "hbs.tmp.image." )
  buf2 = mktemp( "hbs.tmp.image." )

  imarith( tmp_fn1, "-", fnave, buf1, pixtype="real", calctype="real",\
           ver- )
  imarith( buf1, "*", (xkey1-xkeyave), buf1, pixtype="real",\
           calctype="real", ver- )
  imarith( tmp_fn2, "-", fnave, buf2, pixtype="real", calctype="real",\
           ver- )
  imarith( buf2, "*", (xkey2-xkeyave), buf2, pixtype="real",\
           calctype="real", ver- )

  imarith( buf1, "+", buf2, beta2, pixtype="real", calctype="real",\
           ver- )

  xkey_tmp = (xkey1-xkeyave)*(xkey1-xkeyave) + (xkey2-xkeyave)*(xkey2-xkeyave)
  imarith( beta2, "/", xkey_tmp, beta2, pixtype="real", calctype="real",\
           ver- )

  imdelete( buf1, ver- )
  imdelete( buf2, ver- )

  if( flag_log == 1 )
  {
    imdelete( tmp_fn1, ver- )
    imdelete( tmp_fn2, ver- )
  }

  #----- make beta1-image  ( y = beta1 + beta2 * x )

  buf1 = mktemp( "hbs.tmp.image." )
  imarith( beta2, "*", xkeyave, buf1, pixtype="real",\
           calctype="real", ver- )
  imarith( fnave, "-", buf1, beta1, pixtype="real",\
           calctype="real", ver- )
  imdelete( buf1, ver- )

  #----- make interpolated images

  buf1 = mktemp( "hbs.tmp.image." )
  imarith( beta2, "*", keyval, buf1, pixtype="real",\
           calctype="real", ver- )
  imarith( beta1, "+", buf1, outfn, pixtype="real",\
           calctype="real", ver- )
  imdelete( buf1, ver- )

  if( flag_log == 1 )
    imfunc( outfn, outfn, function="alog10", ver- )

  imdelete( beta1, ver- )
  imdelete( beta2, ver- )
  imdelete( fnave, ver- )

end
